mod model;

use std::thread::sleep;
use std::time::{Duration, Instant};
use color_eyre::eyre::ContextCompat;
use color_eyre::Result;
use rgb::RGB8;
use loupedeck_serial::commands::VibrationPattern;
use loupedeck_serial::device::LoupedeckDevice;
use loupedeck_serial::events::LoupedeckEvent;

fn main() -> Result<()> {
    let available_devices = LoupedeckDevice::discover()?;

    let device = available_devices.first()
        .wrap_err("at least one device should be connected")?
        .connect()?;

    println!("Version: {}\nSerial number: {}", device.firmware_version(), device.serial_number());
    device.set_brightness(1.0);

    // run_vibrations(&device)?;
    run_rainbow(&device)?;

    Ok(())
}

fn run_rainbow(device: &LoupedeckDevice) -> Result<()> {
    let interval = Duration::from_millis(50);
    let start = Instant::now();
    let mut iteration = 0;

    let buttons = device.characteristics().available_buttons.iter().filter(|b| b.supports_color()).collect::<Vec<_>>();

    'outer: loop {
        let ms = start.elapsed().as_millis() as u64;

        if !device.events_channel().is_empty() {
            for event in device.events_channel() {
                if let LoupedeckEvent::Disconnected = event {
                    break 'outer;
                }

                if device.events_channel().is_empty() {
                    break;
                }
            }
        }

        for (index, button) in buttons.iter().enumerate() {
            let t = (ms + (index * 100) as u64) as f32;

            device.set_button_color(*button, RGB8::new(
                (((t / 1000.0).sin() / 2.0 + 0.5) * 255.0) as u8,
                (((t / 500.0).sin() / 2.0 + 0.5) * 255.0) as u8,
                (((t / 250.0).sin() / 2.0 + 0.5) * 255.0) as u8,
            ))?;
        }

        sleep((iteration + 1) * interval - start.elapsed());
        iteration += 1;
    };

    Ok(())
}

fn run_vibrations(device: &LoupedeckDevice) -> Result<()> {
    for event in device.events_channel() {
        if let LoupedeckEvent::Touch { is_end: false, .. } = event { device.vibrate(VibrationPattern::Low) }
    }

    Ok(())
}